@@ -1,16 +1,15 @@
 from copy import copy
 
 from .position import Position
-from .excerpt import Excerpt
-from .errors import ParseError
+from .excerpt import PointExcerpt, RangeExcerpt
 
 
 class Buffer:
 
-    def __init__(self, text: str):
+    def __init__(self, text: str, source: str = ""):
         self.text = text
         self.length = len(text)
-
+        self.source = source
 
         self.line_indices = []
         self.line_indentations = []
@@ -66,12 +65,15 @@ class Buffer:
 
         index, line, column = position.index, position.line, position.column
 
+        if index == self.length and line == -1 and column == -1:
+            return True
+
         if index > self.length or line > self.line_count:
             return False
         
         start = self.line_indices[line - 1]
         if line == self.line_count and column == -1:
-            return index == self.length
+            return index == self.length - 1
         
         if column == -1:
             if line == self.line_count:
@@ -122,7 +124,7 @@ class Buffer:
 
             if self.position.index == self.length:
                 self.position.column = -1
-                self.position.line = self.line_count
+                self.position.line = -1
             elif self.text[self.position.index] == "\n":
                 self.position.column = -1
     
@@ -270,9 +272,11 @@ class Buffer:
             the given line's text
         """
 
-        # Set line number to current if zero or less
-        if line_number <= 0:
+        # Set line number to current if zero, or last line if -1
+        if line_number == 0:
             line_number = self.position.line
+        elif line_number == -1:
+            line_number = self.line_count
 
         # Assert line number in range
         if line_number < 1 or line_number > self.line_count:
@@ -307,12 +311,69 @@ class Buffer:
             the given line's indentation
         """
 
-        # Set line to current if not specified
-        if line_number <= 0:
+        # Set line number to current if zero, or last line if -1
+        if line_number == 0:
             line_number = self.position.line
+        elif line_number == -1:
+            line_number = self.line_count
 
         # Check line is in bounds
         if line_number < 1 or line_number > self.line_count:
             return 0
 
-        return self.line_indentations[line_number - 1]
\ No newline at end of file
+        return self.line_indentations[line_number - 1]
+    
+    def point_excerpt(self, position: Position = None) -> PointExcerpt:
+        """ Creates a point excerpt
+
+        If no position is given, the current buffer position is inferred
+        
+        Arguments
+        ---------
+        position: Position
+            the position of the excerpt
+        
+        Returns
+        -------
+        excerpt: PointExcerpt
+            the excerpt created
+        
+        Raises
+        ------
+        position_invalid: ValueError
+            if the position given is invalid
+        """
+
+        if not position:
+            position = self.copy_position()
+        return PointExcerpt(self, position)
+    
+    def range_excerpt(self, start: Position, end: Position = None):
+        """ Creates a range excerpt
+
+        If no end position is given, the current buffer position is inferred
+        
+        Arguments
+        ---------
+        start: Position
+            position of the excerpt start
+        end: Position
+            position of the excerpt end
+        
+        Returns
+        -------
+        excerpt: RangeExcerpt
+            the excerpt created
+        
+        Raises
+        ------
+        position_invalid: ValueError
+            if one of the positions given is invalid
+        position_illogical: IndexError
+            if the range given (or inferred) doesn't make sense, ie: the start
+            position succeeds the end
+        """
+        
+        if not end:
+            end = self.copy_position()
+        return RangeExcerpt(self, start, end)
\ No newline at end of file
